package xyz.tbvns.Editor;

import com.badlogic.gdx.graphics.g3d.Model;
import com.badlogic.gdx.graphics.g3d.ModelInstance;
import lombok.Getter;
import xyz.tbvns.Models.Animation;
import xyz.tbvns.Models.GDModel;
import xyz.tbvns.Models.OBJReader;
import xyz.tbvns.Physics.BulletManager;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

public class ModelsManager {
    public static final List<GDModel> models = new ArrayList<>();
    public static final List<Animation> animations = new ArrayList<>();

    @Getter
    private static Model selectedModel;
    @Getter
    private static ModelInstance selectedModelInstance;
    @Getter
    private static List<ModelInstance> loadedModels = new ArrayList<>();

    public static void addModel(File obj) throws FileNotFoundException {
        File mtl = new File(obj.getParent() + "/" + obj.getName().replace(".obj", ".mtl"));
        System.out.println(mtl.getPath());
        if (!mtl.exists()) {
            throw new FileNotFoundException("Missing mtl file");
        }

        GDModel model = new GDModel(
                OBJReader.getFaces(obj, mtl),
                obj.getName().replace(".obj", "")
        );

        models.add(model);
    }

    public static void addAnimated(File folder) throws FileNotFoundException {
        List<File> OBJs = new ArrayList<>();
        List<File> MTLs = new ArrayList<>();

        for (int i = 0; i < folder.listFiles().length; i++) {
            File f = folder.listFiles()[i];
            if (f.getAbsolutePath().endsWith(".obj")) {
                OBJs.add(f);
            } else
            if (f.getAbsolutePath().endsWith(".mtl")) {
                MTLs.add(f);
            }
        }

        Animation animation = new Animation();

        JFrame frame = new JFrame("Loading...");
        frame.setContentPane(new JPanel() {{
            add(new JLabel("Loading animation " + folder.getName()));
        }});
        frame.setSize(300, 50);
        frame.setLocation(Toolkit.getDefaultToolkit().getScreenSize().width / 2 - 150, Toolkit.getDefaultToolkit().getScreenSize().height / 2 - 25);
        frame.setVisible(true);

        for (int i = 0; i < OBJs.size(); i++) {
            GDModel model = new GDModel(
                    OBJReader.getFaces(OBJs.get(i), MTLs.get(i)),
                    folder.getName()
            );
            animation.getFrames().add(model);
            animation.setName(folder.getName());
        }
        animations.add(animation);
        frame.setVisible(false);
    }

    public static void select(GDModel model) {
        selectedModel = model.toModel();
        selectedModelInstance = new ModelInstance(selectedModel);
        selectedModelInstance.userData = model;
        BulletManager.addModel(selectedModelInstance);
        loadedModels.add(selectedModelInstance);
    }

    public static void select(ModelInstance model) {
        selectedModelInstance = model;
        selectedModel = model.model;
    }

    public static void deselect() {
        selectedModelInstance = null;
        selectedModel = null;
    }
}
